@@ -40,9 +40,10 @@ import (
 )
 
 const (
-	hostErrorRetryDelay    = time.Second * 10
-	pauseRetryDelay        = time.Second * 30
-	rebootAnnotationPrefix = "reboot.metal3.io"
+	hostErrorRetryDelay           = time.Second * 10
+	unmanagedRetryDelay           = time.Minute * 10
+	provisionerNotReadyRetryDelay = time.Second * 30
+	rebootAnnotationPrefix        = "reboot.metal3.io"
 )
 
 var runInTestMode bool
@@ -195,7 +196,8 @@ func (r *ReconcileBareMetalHost) Reconcile(request reconcile.Request) (result re
 	annotations := host.GetAnnotations()
 	if annotations != nil {
 		if _, ok := annotations[metal3v1alpha1.PausedAnnotation]; ok {
-			return reconcile.Result{Requeue: true, RequeueAfter: pauseRetryDelay}, nil
+			reqLogger.Info("host is paused, no work to do")
+			return reconcile.Result{Requeue: false}, nil
 		}
 	}
 
@@ -257,14 +259,21 @@ func (r *ReconcileBareMetalHost) Reconcile(request reconcile.Request) (result re
 	// Retrieve the BMC details from the host spec and validate host
 	// BMC details and build the credentials for talking to the
 	// management controller.
-	bmcCreds, bmcCredsSecret, err := r.buildAndValidateBMCCredentials(request, host)
-	if err != nil || bmcCreds == nil {
-		if !host.DeletionTimestamp.IsZero() {
-			// If we are in the process of deletion, try with empty credentials
-			bmcCreds = &bmc.Credentials{}
-			bmcCredsSecret = &corev1.Secret{}
-		} else {
-			return r.credentialsErrorResult(err, request, host)
+	var bmcCreds *bmc.Credentials
+	var bmcCredsSecret *corev1.Secret
+	switch host.Status.Provisioning.State {
+	case metal3v1alpha1.StateNone, metal3v1alpha1.StateUnmanaged:
+		bmcCreds = &bmc.Credentials{}
+	default:
+		bmcCreds, bmcCredsSecret, err = r.buildAndValidateBMCCredentials(request, host)
+		if err != nil || bmcCreds == nil {
+			if !host.DeletionTimestamp.IsZero() {
+				// If we are in the process of deletion, try with empty credentials
+				bmcCreds = &bmc.Credentials{}
+				bmcCredsSecret = &corev1.Secret{}
+			} else {
+				return r.credentialsErrorResult(err, request, host)
+			}
 		}
 	}
 
@@ -280,6 +289,16 @@ func (r *ReconcileBareMetalHost) Reconcile(request reconcile.Request) (result re
 		return reconcile.Result{}, errors.Wrap(err, "failed to create provisioner")
 	}
 
+	ready, err := prov.IsReady()
+
+	if err != nil {
+		return reconcile.Result{}, errors.Wrap(err, "failed to check services availability")
+	}
+	if !ready {
+		reqLogger.Info("provisioner is not ready", "RequeueAfter:", provisionerNotReadyRetryDelay)
+		return reconcile.Result{Requeue: true, RequeueAfter: provisionerNotReadyRetryDelay}, nil
+	}
+
 	stateMachine := newHostStateMachine(host, r, prov)
 	actResult := stateMachine.ReconcileState(info)
 	result, err = actResult.Result()
@@ -351,30 +370,6 @@ func recordActionFailure(info *reconcileInfo, errorType metal3v1alpha1.ErrorType
 
 func (r *ReconcileBareMetalHost) credentialsErrorResult(err error, request reconcile.Request, host *metal3v1alpha1.BareMetalHost) (reconcile.Result, error) {
 	switch err.(type) {
-	// We treat an empty bmc address and empty bmc credentials fields as a
-	// trigger the host needs to be put into a discovered status. We also set
-	// an error message (but not an error state) on the host so we can understand
-	// what we may be waiting on.  Editing the host to set these values will
-	// cause the host to be reconciled again so we do not Requeue.
-	case *EmptyBMCAddressError, *EmptyBMCSecretError:
-		credentialsInvalid.Inc()
-		dirty := host.SetOperationalStatus(metal3v1alpha1.OperationalStatusDiscovered)
-		if dirty {
-			// Set the host error message directly
-			// as we cannot use SetErrorCondition which
-			// overwrites our discovered state
-			host.Status.ErrorMessage = err.Error()
-			host.Status.ErrorType = ""
-			saveErr := r.saveHostStatus(host)
-			if saveErr != nil {
-				return reconcile.Result{Requeue: true}, saveErr
-			}
-			// Only publish the event if we do not have an error
-			// after saving so that we only publish one time.
-			r.publishEvent(request,
-				host.NewEvent("Discovered", fmt.Sprintf("Discovered host with unusable BMC details: %s", err.Error())))
-		}
-		return reconcile.Result{}, nil
 	// In the event a credential secret is defined, but we cannot find it
 	// we requeue the host as we will not know if they create the secret
 	// at some point in the future.
@@ -390,12 +385,14 @@ func (r *ReconcileBareMetalHost) credentialsErrorResult(err error, request recon
 			r.publishEvent(request, host.NewEvent("BMCCredentialError", err.Error()))
 		}
 		return reconcile.Result{Requeue: true, RequeueAfter: hostErrorRetryDelay}, nil
-	// If we have found the secret but it is missing the required fields
-	// or the BMC address is defined but malformed we set the
+	// If a managed Host is missing a BMC address or secret, or
+	// we have found the secret but it is missing the required fields,
+	// or the BMC address is defined but malformed, we set the
 	// host into an error state but we do not Requeue it
 	// as fixing the secret or the host BMC info will trigger
 	// the host to be reconciled again
-	case *bmc.CredentialsValidationError, *bmc.UnknownBMCTypeError:
+	case *EmptyBMCAddressError, *EmptyBMCSecretError,
+		*bmc.CredentialsValidationError, *bmc.UnknownBMCTypeError:
 		credentialsInvalid.Inc()
 		_, saveErr := r.setErrorCondition(request, host, metal3v1alpha1.RegistrationError, err.Error())
 		if saveErr != nil {
@@ -456,10 +453,6 @@ func (r *ReconcileBareMetalHost) actionDeleting(prov provisioner.Provisioner, in
 		return actionError{errors.Wrap(err, "failed to delete")}
 	}
 	if provResult.Dirty {
-		err = r.saveHostStatus(info.host)
-		if err != nil {
-			return actionError{errors.Wrap(err, "failed to save host after deleting")}
-		}
 		return actionContinue{provResult.RequeueAfter}
 	}
 
@@ -475,6 +468,13 @@ func (r *ReconcileBareMetalHost) actionDeleting(prov provisioner.Provisioner, in
 	return deleteComplete{}
 }
 
+func (r *ReconcileBareMetalHost) actionUnmanaged(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
+	if info.host.HasBMCDetails() {
+		return actionComplete{}
+	}
+	return actionContinueNoWrite{actionContinue{unmanagedRetryDelay}}
+}
+
 // Test the credentials by connecting to the management controller.
 func (r *ReconcileBareMetalHost) actionRegistering(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 	info.log.Info("registering and validating access to management controller",
@@ -588,6 +588,7 @@ func (r *ReconcileBareMetalHost) actionMatchProfile(prov provisioner.Provisioner
 		info.log.Info("updating hardware profile", "profile", hardwareProfile)
 		info.publishEvent("ProfileSet", fmt.Sprintf("Hardware profile set: %s", hardwareProfile))
 	}
+
 	info.host.ClearError()
 	return actionComplete{}
 }
@@ -637,6 +638,13 @@ func (r *ReconcileBareMetalHost) actionProvisioning(prov provisioner.Provisioner
 	return actionComplete{}
 }
 
+// clearHostProvisioningSettings removes the values related to
+// provisioning that do not trigger re-provisioning from the status
+// fields of a host.
+func clearHostProvisioningSettings(host *metal3v1alpha1.BareMetalHost) {
+	host.Status.Provisioning.RootDeviceHints = nil
+}
+
 func (r *ReconcileBareMetalHost) actionDeprovisioning(prov provisioner.Provisioner, info *reconcileInfo) actionResult {
 	info.log.Info("deprovisioning")
 
@@ -661,9 +669,10 @@ func (r *ReconcileBareMetalHost) actionDeprovisioning(prov provisioner.Provision
 		return actionContinueNoWrite{}
 	}
 
-	// After the provisioner is done, clear the image settings so we
-	// transition to the next state.
+	// After the provisioner is done, clear the provisioning settings
+	// so we transition to the next state.
 	info.host.Status.Provisioning.Image = metal3v1alpha1.Image{}
+	clearHostProvisioningSettings(info.host)
 
 	return actionComplete{}
 }
@@ -712,7 +721,7 @@ func (r *ReconcileBareMetalHost) manageHostPower(prov provisioner.Provisioner, i
 	// a delay.
 	steadyStateResult := actionContinue{time.Second * 60}
 	if info.host.Status.PoweredOn == desiredPowerOnState {
-		return steadyStateResult
+		return actionContinueNoWrite{steadyStateResult}
 	}
 
 	info.log.Info("power state change needed",
@@ -800,13 +809,46 @@ func (r *ReconcileBareMetalHost) actionManageReady(prov provisioner.Provisioner,
 	}
 
 	if info.host.NeedsProvisioning() {
+		// Ensure the provisioning settings we're going to use are stored.
+		dirty, err := saveHostProvisioningSettings(info.host)
+		if err != nil {
+			return actionError{errors.Wrap(err, "Could not save the host provisioning settings")}
+		}
+		if dirty {
+			info.log.Info("updating host provisioning settings")
+		}
 		info.host.ClearError()
 		return actionComplete{}
 	}
 	return r.manageHostPower(prov, info)
 }
 
-func (r *ReconcileBareMetalHost) saveHostStatus(host *metal3v1alpha1.BareMetalHost) error {
+// saveHostProvisioningSettings copies the values related to
+// provisioning that do not trigger re-provisioning into the status
+// fields of the host.
+func saveHostProvisioningSettings(host *metal3v1alpha1.BareMetalHost) (dirty bool, err error) {
+
+	// Ensure the root device hints we're going to use are stored.
+	//
+	// If the user has provided explicit root device hints, they take
+	// precedence. Otherwise use the values from the hardware profile.
+	hintSource := host.Spec.RootDeviceHints
+	if hintSource == nil {
+		hwProf, err := hardware.GetProfile(host.HardwareProfile())
+		if err != nil {
+			return false, errors.Wrap(err, "Could not update root device hints")
+		}
+		hintSource = &hwProf.RootDeviceHints
+	}
+	if (hintSource != nil && host.Status.Provisioning.RootDeviceHints == nil) || *hintSource != *(host.Status.Provisioning.RootDeviceHints) {
+		host.Status.Provisioning.RootDeviceHints = hintSource
+		dirty = true
+	}
+
+	return
+}
+
+func (r *ReconcileBareMetalHost) saveHostStatus(host *metal3v1alpha1.BareMetalHost) (err error) {
 	t := metav1.Now()
 	host.Status.LastUpdated = &t
 
